﻿using Microsoft.AspNet.Builder;
using Microsoft.AspNet.Http;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace WebAPIDemo_1.Middlewares
{
    public class ETagMiddleware
    {
        private RequestDelegate _next;
        HttpContext _httpContext;

        public ETagMiddleware(RequestDelegate next)
        {
            _next = next;
        }

        public async Task Invoke(HttpContext context)
        {
            this._httpContext = context;

            context.Response.OnStarting(OnStartingCallback, state: this);

            await _next(context);
        }

        public Task OnStartingCallback(object state)
        {
            var request = _httpContext.Request;

            var key = GetKey();
            var keyBytes = Encoding.UTF8.GetBytes(key);

            if (request.Body.CanSeek)
            {
                request.Body.Position = 0;
            }
            var input = new StreamReader(request.Body).ReadToEnd();
            var contentBytes = Encoding.UTF8.GetBytes(input);

            var combinedBytes = CombineBytes(keyBytes, contentBytes);
            var ETag = GenerateETag(combinedBytes);

            if (request.Method == "GET")
            {
                if (request.Headers.Keys.Contains("If-None-Match") && request.Headers["If-None-Match"].ToString() == ETag)
                {
                    // Not Modified
                    _httpContext.Response.StatusCode = 304;
                    return Task.FromResult(0);
                }

                _httpContext.Response.Headers.Add("ETag", new[] { ETag });
            }

            return Task.FromResult(0);
        }

        private string GenerateETag(byte[] data)
        {
            string tag = string.Empty;

            using (var md5 = MD5.Create())
            {
                var hash = md5.ComputeHash(data);
                string hex = BitConverter.ToString(hash);
                tag = hex.Replace("-", "");
            }

            return tag;
        }

        byte[] CombineBytes(byte[] first, byte[] second)
        {
            byte[] combinedBytes = new byte[first.Length + second.Length];
            Buffer.BlockCopy(first, 0, combinedBytes, 0, first.Length);
            Buffer.BlockCopy(second, 0, combinedBytes, first.Length, second.Length);
            return combinedBytes;
        }

        private string GetKey()
        {
            return _httpContext.Request.Path.ToString();
        }
    }
}
