﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNet.Builder;
using Microsoft.AspNet.Hosting;
using Microsoft.AspNet.Http;
using Microsoft.Extensions.DependencyInjection;
using WebAPITest.Data.Models;
using Microsoft.AspNet.Mvc;
using Microsoft.AspNet.Mvc.Formatters.Xml;
using Newtonsoft.Json.Serialization;
using Newtonsoft.Json;
using Microsoft.AspNet.Identity.EntityFramework;

using Microsoft.Extensions.Configuration;
using Microsoft.AspNet.Authorization;
using Microsoft.AspNet.Mvc.Filters;
using WebAPIDemo_1.Middlewares;

namespace WebAPIDemo_1
{
    public class Startup
    {
        // This method gets called by the runtime. Use this method to add services to the container.
        // For more information on how to configure your application, visit http://go.microsoft.com/fwlink/?LinkID=398940
        public void ConfigureServices(IServiceCollection services)
        {
            var defaultPolicy = new AuthorizationPolicyBuilder()
               .RequireAuthenticatedUser()
               .Build();

            var mvcBuilder = services.AddMvc(setup =>
            {
               // setup.Filters.Add(new AuthorizeFilter(defaultPolicy));
            });


            services.AddAuthorization(options =>
            {
                // inline policies
                options.AddPolicy("AllowUpdate", policy =>
                {
                    policy.RequireClaim("CanUpdate", "true");
                });
                options.AddPolicy("AllowDelete", policy =>
                {
                    policy.RequireClaim("CanDelete", "true");
                });
            });


            mvcBuilder.AddJsonOptions(options =>
             {
                 options.SerializerSettings.ReferenceLoopHandling = ReferenceLoopHandling.Ignore;
                 options.SerializerSettings.ContractResolver = new CamelCasePropertyNamesContractResolver();                            
             }
            );

            mvcBuilder.AddXmlDataContractSerializerFormatters();

            services.AddEntityFramework().AddSqlServer().AddDbContext<BookStoreContext>();
       
            services.AddTransient<BookContextSeeder>();
            services.AddScoped<IBookStoreRepository, BookStoreRepository>();

            services.AddIdentity<ApplicationUser, IdentityRole>()
             .AddEntityFrameworkStores<BookStoreContext>();

        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, BookContextSeeder seeder)
        {
            app.UseIdentity();
            app.UseMiddleware<ETagMiddleware>();
            app.UseMvc(routes =>
            {
                routes.MapRoute(
                    name: "default",
                    template: "{controller=Account}/{action=Login}");
            });

            seeder.Seed();
        }

        // Entry point for the application.
        public static void Main(string[] args) => WebApplication.Run<Startup>(args);
    }
}
