﻿using Microsoft.AspNet.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using WebAPITest.Data.Models;

namespace WebAPIDemo_1.Controllers
{
    [Route("api/[controller]")]
    public class BooksController : Controller
    {
        private IBookStoreRepository repository;

        public BooksController(IBookStoreRepository _repository)
        {
            this.repository = _repository;
        }

        [HttpGet]
        public IEnumerable<Book> Get()
        {
            return this.repository.GetAllBooks();
        }

        [HttpGet("{id:int}/{includeAuthor:bool?}", Name ="GetBooks")]
        public Book Get(int Id, bool includeAuthor = false)
        {
            return this.repository.FindBook(Id, includeAuthor);
        }

        [HttpGet("{id:int}/author")]
        public Author Get(int id)
        {
            return this.repository.FindBook(id, true).Author;
        }

        [HttpPost]
        public IActionResult Post([FromBody]Book newBook)
        {
            if(newBook == null)
            {
                return HttpBadRequest("Data is not received in proper format");
            }

            var addedBook = this.repository.AddBook(newBook);

            if (addedBook == null)
            {
                return HttpBadRequest("Error while adding a book");
            }


            return CreatedAtRoute("GetBooks", new { controller = "Books", id = addedBook.Id }, addedBook);

        }

        [HttpPut("{id:int}")]
        public IActionResult Put(int id, [FromBody] Book book)
        {
            if (book == null)
            {
                return HttpBadRequest("Data is not received in proper format");
            }

            bool bookUpdated = this.repository.UpdateBook(book);

            if (!bookUpdated)
            {
                return HttpBadRequest("Error while updating the book");
            }
             
            return new HttpOkResult();
        }

        [HttpDelete("{id:int}")]
        public IActionResult Delete(int id)
        {
            if (id == 0)
            {
                return HttpBadRequest("Data is not received in proper format");
            }

            var book = this.repository.FindBook(id, false);

            if(book == null)
            {
                return new HttpNotFoundResult();
            }

            bool isDeleted = this.repository.DeleteBook(id);

            if (!isDeleted)
            {
                return HttpBadRequest("Error while deleting the book");
            }

            return new HttpOkResult();
        }

        [HttpPatch("{id:int}")]
        public IActionResult Patch(int id, [FromBody] Book book)
        {
            if (book == null)
            {
                return HttpBadRequest("Data is not received in proper format");
            }

            bool bookUpdated = this.repository.UpdateBook(book);

            if (!bookUpdated)
            {
                return HttpBadRequest("Error while updating the book");
            }
            

            return new HttpOkResult();
        }

        //public IEnumerable<string> Get()
        //{
        //    return new List<string>() { "Brij", "John" };
        //}
    }
}
