﻿using Microsoft.AspNet.Http;
using Microsoft.AspNet.Mvc.ActionConstraints;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace WebAPIDemo_1.Versioning
{
    public class VersionRangeValidator : IActionConstraint
    {
        private readonly int? _minVersion;
        private readonly int? _maxVersion;

        public int Order { get; set; }

        public VersionRangeValidator(int? minVersion, int? maxVersion)
        {
            _minVersion = minVersion;
            _maxVersion = maxVersion;
        }

        public static string GetVersion(HttpRequest request)
        {
            if (!string.IsNullOrWhiteSpace(request.Headers["X-BookStoreAPI-Version"]))
                return request.Headers["X-BookStoreAPI-Version"];

            return "1";
        }

        public bool Accept(ActionConstraintContext context)
        {
            int version;
            if (int.TryParse(GetVersion(context.RouteContext.HttpContext.Request), out version))
            {
                return (_minVersion == null || _minVersion <= version) &&
                    (_maxVersion == null || _maxVersion >= version);
            }
            else
            {
                return false;
            }
        }
    }
}
