﻿using Microsoft.AspNet.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;
using Newtonsoft.Json;
using WebAPIDemo_1.Models;
using WebAPITest.Data.Models;
using WebAPIDemo_1.Versioning;

namespace WebAPIDemo_1.Controllers
{
    [Route("api/Books")]
    public class BooksV2Controller : Controller
    {

        private IBookStoreRepository repository;
        private IUrlHelper urlHelper;
        const int maxPageSize = 8;

        public BooksV2Controller(IBookStoreRepository _repository, IUrlHelper urlHelper)
        {
            this.repository = _repository;
            this.urlHelper = urlHelper;
        }


        //[DisableCors]
        //[HttpGet(Name = "GetBooks")]
        public IEnumerable<BookV2> Get(string title = null,
            string isbn = null,
            string genre = null,
            string sort = "Id",
            string order = "Asc",
            int pageNo = 1,
            int pageSize = maxPageSize)
        {
            IList<Book> allbooks = this.repository.GetAllBooks();

            // Reset page size to max page size if requested page size is greater than max page size
            if (pageSize > maxPageSize)
            {
                pageSize = maxPageSize;
            }

            // Search on the string

            allbooks = allbooks.Where(b => (title == null || b.Title.Contains(title)))
                .Where(b => (isbn == null || b.ISBN.Replace("-", string.Empty) == isbn.Replace("-", string.Empty)))
                .Where(b => (genre == null || b.Genre == genre)).ToList();


            // Implement sorting
            IList<Book> sortedBooks = allbooks.ApplySorting(sort, order);

            // Creating metadata for paging
            int totalBooks = sortedBooks.Count;
            int totalPages = (int)Math.Ceiling((double)totalBooks / pageSize);

            var prevPageLink = pageNo == 1 ? string.Empty : this.urlHelper.Link("GetBooks",
                new
                {
                    sort = sort,
                    order = order,
                    pageNo = pageNo - 1,
                    pageSize = pageSize
                });

            var nextPageLink = pageNo == totalPages ? string.Empty : this.urlHelper.Link("GetBooks",
                new
                {
                    sort = sort,
                    order = order,
                    pageNo = pageNo + 1,
                    pageSize = pageSize
                });

            // page header info
            var pageInfoHeader = new
            {
                pageNo = pageNo,
                pageSize = pageSize,
                totalbooks = totalBooks,
                totalPages = totalPages,
                prevPageLink = prevPageLink,
                nextPageLink = nextPageLink
            };

            // adding page details in header
            Response.Headers.Add("X-PageInfo", JsonConvert.SerializeObject(pageInfoHeader));

            var sortedBooksV2 = sortedBooks
                .Select(b => new BookV2(b))
                .ToList();
            // 
            return sortedBooksV2.Skip((pageNo - 1) * pageSize).Take(pageSize);
        }

        [VersionGet("{id:int}/{includeAuthor:bool?}", versionRange: "[2-4]")]
        public BookV2 Get(int Id, bool includeAuthor = false)
        {
            return new BookV2(this.repository.FindBook(Id, includeAuthor));
        }

    }
}
